"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const pino_1 = require("../pino");
const __1 = require("../..");
const express_1 = __importDefault(require("express"));
const express_session_1 = __importDefault(require("express-session"));
const passport_1 = __importDefault(require("passport"));
const passport_google_oauth20_1 = require("passport-google-oauth20");
const router_1 = require("./router");
const models_1 = require("../../models");
const branch_resolver_1 = require("../../middleware/branch-resolver");
const user_resolver_1 = require("../../middleware/user-resolver");
const body_parser_1 = __importDefault(require("body-parser"));
const cookie_parser_1 = __importDefault(require("cookie-parser"));
const http_1 = __importDefault(require("http"));
const pino_http_1 = __importDefault(require("pino-http"));
// eslint-disable-next-line @typescript-eslint/no-var-requires
const KnexSessionStore = require('connect-session-knex')(express_session_1.default);
const uuid_1 = require("uuid");
const sessionSecret = process.env.SESSION_SECRET;
if (!sessionSecret) {
    throw new Error('Session secret is required');
}
const googleClientId = process.env.GOOGLE_CLIENT_ID;
const googleClientSecret = process.env.GOOGLE_CLIENT_SECRET;
if (!googleClientId || !googleClientSecret) {
    throw new Error('Google client id and secret are required');
}
passport_1.default.use(new passport_google_oauth20_1.Strategy({
    clientID: googleClientId,
    clientSecret: googleClientSecret,
    callbackURL: `${__1.config.appHost}/auth/google/callback`,
}, (accessToken, refreshToken, profile, cb) => {
    models_1.User.findOrCreateByGoogleId(profile.id)
        .then((id) => {
        return cb(undefined, { id });
    })
        .catch((error) => {
        return cb(error, undefined);
    });
}));
passport_1.default.serializeUser((user, done) => {
    done(null, user);
});
passport_1.default.deserializeUser((user, done) => {
    done(null, user);
});
function stringifyHeaders(headers) {
    if (!process.env.DEV_LOG_HEADERS) {
        return '';
    }
    return ('-- ' +
        Object.keys(headers)
            .map((name) => {
            return `${name}: ${headers[name]}`;
        })
            .join(' • '));
}
function generateServer(appCtx, routes) {
    const httpLogger = pino_http_1.default({
        logger: pino_1.baseLogger,
        genReqId: uuid_1.v4,
        serializers: {
            req: (req) => {
                if (appCtx.devMode) {
                    return `${req.method} ${req.url} ${stringifyHeaders(req.headers)}`.trim();
                }
                else {
                    return {
                        id: req.id,
                        method: req.method,
                        url: req.url,
                        headers: req.headers,
                    };
                }
            },
            res: (res) => {
                if (appCtx.devMode) {
                    return res.statusCode;
                }
                else {
                    return { statusCode: res.statusCode };
                }
            },
        },
    });
    const expressCtx = appCtx.create('server');
    const app = express_1.default();
    const router = express_1.default.Router(); // eslint-disable-line new-cap
    app.disable('x-powered-by');
    app.use(httpLogger);
    app.use((req, res, next) => {
        req.ctx = new __1.Context(`${req.method} ${req.path}`, {}, appCtx, req.log);
        next();
    });
    app.use(__1.sentry.Handlers.requestHandler());
    app.use(body_parser_1.default.json());
    app.use(cookie_parser_1.default());
    app.use(express_session_1.default({
        name: 'scp_session',
        cookie: {
            maxAge: 1000 * 60 * 60 * 24 * 30 * 2,
        },
        secret: sessionSecret,
        store: new KnexSessionStore({ knex: models_1.db }),
        resave: false,
        saveUninitialized: false,
    }));
    app.use(passport_1.default.initialize());
    app.use(passport_1.default.session());
    app.use(branch_resolver_1.branchResolverMiddleware);
    app.use(user_resolver_1.userResolverMiddleware);
    app.get('/auth/google', (req, res) => {
        const returnPath = req.query.returnPath;
        if (returnPath && returnPath.startsWith('/') && req.session) {
            req.returnPath = returnPath;
        }
        passport_1.default.authenticate('google', { scope: ['profile'] })(req, res);
    });
    app.get('/auth/google/callback', passport_1.default.authenticate('google', { failureRedirect: '/' }), (req, res) => {
        res.redirect(req.returnPath || '/');
    });
    app.get('/auth/logout', (req, res) => {
        req.logout();
        res.redirect('/');
    });
    app.get('/auth/whoami', (req, res) => {
        if (req.userInfo) {
            const { id, username, wikidotUid } = req.userInfo;
            res.send({ id, username, wikidotUid });
        }
        else {
            res.send({ id: null });
        }
    });
    router_1.initRoutes(app, routes);
    app.use('/', router);
    app.use(__1.sentry.Handlers.errorHandler());
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    app.use((err, req, res, next) => {
        if (req.ctx) {
            req.ctx.logError('Unhandled error', err);
        }
        else {
            console.error('UNHANDLED ERROR', err);
        }
        res.status(500).send('Internal server error');
    });
    return function listen(port) {
        const server = http_1.default.createServer(app);
        server.on('listening', () => {
            expressCtx.log(`Express server is listening on port ${server.address().port}`);
        });
        server.on('error', (error) => {
            if (error.syscall !== 'listen') {
                throw error;
            }
            switch (error.code) {
                case 'EACCES':
                    expressCtx.logError(`Port ${port} requires elevated privileges`, error);
                    process.exit(1);
                    break;
                case 'EADDRINUSE':
                    expressCtx.logError(`Port ${port} is already in use`, error);
                    process.exit(1);
                    break;
                default:
                    throw error;
            }
        });
        server.listen(port);
    };
}
exports.default = generateServer;
//# sourceMappingURL=index.js.map