"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.run = void 0;
const lodash_1 = require("lodash");
const models_1 = require("../models");
const vote_1 = require("../models/vote");
const PAGE_NAME_BLACKLIST = /^(admin|nav|css|system):/;
function run(ctx, branchName, pageName) {
    return __awaiter(this, void 0, void 0, function* () {
        const { Branch, Page, PageRevision, PageComment } = ctx;
        if (!(branchName && pageName)) {
            throw new Error('Wiki name & page name are required');
        }
        const branch = yield Branch.query().where({ name: branchName }).first();
        if (!branch) {
            throw new Error(`Unknown site "${branchName}"`);
        }
        const wikiUrl = branch.url;
        if (PAGE_NAME_BLACKLIST.test(pageName)) {
            return;
        }
        ctx.log(`Importing page ${branchName}/${pageName}`);
        const tx = yield ctx.db.transaction();
        try {
            const page = yield ctx.wk.fetchPage(branchName, pageName);
            ctx.log('Fetched page');
            const pageRevisions = yield ctx.wk.fetchPageRevisionsList(branch.url, pageName);
            const currentRevision = pageRevisions[0];
            const currentRevisionContent = yield ctx.wk.fetchPageRevisionContent(branch.url, currentRevision.id);
            ctx.log('Fetched last revision content');
            const votes = yield ctx.wk.fetchPageVotes(wikiUrl, pageName);
            ctx.log('Fetched votes', { extra: { votesAmount: votes.length } });
            ctx.log('Fetching author profile', { createdBy: page.created_by });
            const pageAuthorProfile = page.created_by
                ? yield ctx.wk.fetchUserProfileByUsername(page.created_by)
                : { uid: -1 };
            ctx.log('Fetching author and updater');
            const pageAuthor = yield models_1.User.fetchOrResolveByUid(ctx, branch.url, pageAuthorProfile.uid);
            const pageUpdater = yield models_1.User.fetchOrResolveByUid(ctx, branch.url, currentRevision.uid);
            ctx.log('Fetched author and updater', { pageAuthor, pageUpdater });
            ctx.log('Preparing page entry');
            let savedPage = yield Page.query(tx).findOne({ branchName, name: pageName });
            if (!savedPage) {
                savedPage = yield Page.query(tx)
                    .insert({
                    branchName,
                    name: pageName,
                    createdById: pageAuthor.id,
                    updatedById: pageUpdater.id,
                    createdAt: new Date(page.created_at),
                    updatedAt: new Date(page.updated_at),
                })
                    .returning('*');
            }
            ctx.log('Preparing page tags', page.tags);
            const pageTags = [];
            for (const tagName of page.tags) {
                let tag = yield models_1.Tag.query(tx).findOne({ branchName, name: tagName });
                if (!tag) {
                    tag = yield models_1.Tag.query(tx).insert({ branchName, name: tagName }).returning('*');
                }
                pageTags.push({ tagId: tag.id, pageId: savedPage.id });
            }
            ctx.log('Inserting page tags');
            yield models_1.PageTag.query(tx).where({ pageId: savedPage.id }).delete();
            yield models_1.PageTag.query(tx).insert(pageTags);
            ctx.log('Preparing page votes');
            const preparedVotes = [];
            for (const vote of votes) {
                if (vote.uid === -1) {
                    continue;
                }
                const voteUser = yield models_1.User.fetchOrResolveByUid(ctx, branch.url, vote.uid);
                preparedVotes.push({
                    value: vote.vote === '+' ? 1 : -1,
                    pageId: savedPage.id,
                    userId: voteUser.id,
                });
            }
            ctx.log('Preparing current revision for the page');
            let currentSavedRevision = yield PageRevision.query(tx).findOne({ id: currentRevision.id });
            if (!currentSavedRevision) {
                const revId = currentRevision.id.toString();
                ctx.log('Creating current revision for the page', { revId });
                currentSavedRevision = yield PageRevision.query(tx).insert({
                    id: revId,
                    pageId: savedPage.id,
                    createdAt: new Date(page.updated_at),
                    createdById: pageUpdater.id,
                    title: page.title,
                    content: currentRevisionContent,
                });
            }
            const currentRevisionId = currentSavedRevision.id;
            ctx.log('Updated current revision id in page', { currentRevisionId });
            yield Page.query(tx).where({ id: savedPage.id }).update({ currentRevisionId });
            ctx.log('Inserting votes');
            yield vote_1.Vote.query(tx).where({ pageId: savedPage.id }).delete();
            yield vote_1.Vote.query(tx).insert(lodash_1.uniqBy(preparedVotes, 'userId'));
            ctx.log('Fetching comments');
            const pageComments = yield ctx.wk.fetchPageComments(branchName, pageName);
            ctx.log(`Fetched ${pageComments.length}`);
            for (const pageComment of pageComments) {
                const currentSavedComment = yield PageComment.query(tx).findOne({ wikidotId: pageComment.id });
                if (!currentSavedComment) {
                    ctx.log('Inserting new comment', { id: pageComment.id });
                    yield PageComment.query(tx).insert({
                        pageId: savedPage.id,
                        wikidotId: pageComment.id,
                        title: pageComment.title,
                        content: pageComment.content,
                        html: pageComment.html,
                        createdAt: new Date(pageComment.created_at),
                        createdBy: pageComment.created_by,
                    });
                }
            }
            ctx.log('Commiting results');
            yield tx.commit();
            ctx.log(`Done: ${branchName}/${pageName}`);
        }
        catch (error) {
            yield tx.rollback();
            ctx.reportError(`Error importing page ${branchName}/${pageName}`, error, { wikiName: branchName, pageName });
            // Remove me
            yield ctx.db.raw('insert into fails (page, error) values (?, ?)', [pageName, error.toString()]);
        }
    });
}
exports.run = run;
//# sourceMappingURL=page-import.js.map